import os
import sys
import traceback
from logging import StreamHandler
from psi_notifier.notifier import Notifier


class PSINotifierLoggingHandler(StreamHandler):
    def __init__(self, notifier: Notifier = None, topic_arn: str = None):
        """Initialize a PSI Notifier logging handler
        If a Notifier object is supplied, this handler will use the provided
        Notifier.
        If a Notifier is not supplied, a topic_arn must be set for this handler
        to initialize it's own Notifier with the provided topic_arn

        Parameters
        ----------
        notifier: Notifier
            A pre-configured PSI Notifier object

        topic_arn: str
            The ARN to publish SNS messages to if notifier is not provided

        Example Usage
        -------------
        logger = logging.getLogger(os.path.basename(__file__))
        logger.setLevel(logging.INFO)
        formatter = logging.Formatter('<%(asctime)s> [%(levelname)s] %(name)s:%(funcName)s -> %(message)s')
        notifier = Notifier(sns_topic_arn='arn:aws:sns:us-east-1:490126044637:jmTest')
        notifier_handler = PSINotifierLoggingHandler(notifier)
        terminal_handler = logging.StreamHandler(sys.stdout)
        notifier_handler.setFormatter(formatter)
        notifier_handler.setLevel(logging.WARNING)
        terminal_handler.setFormatter(formatter)
        logger.addHandler(notifier_handler)
        logger.addHandler(terminal_handler)
        logger.info('This will only log to stdout')
        logger.warning('This will  log to SNS')
        """
        StreamHandler.__init__(self)
        if not notifier and not topic_arn:
            raise Exception('Must provide either a Notifier or SNS topic ARN!')
        if notifier and not topic_arn:
            self.notifier = notifier
        if topic_arn and not notifier:
            self.notifier = Notifier(sns_topic_arn=topic_arn)
        if topic_arn and notifier:
            self.notifier = Notifier(sns_topic_arn=topic_arn)

    def emit(self, record):
        message = self.format(record)
        self.notifier.notify(message, f'[{record.levelname}] {record.filename}:{record.funcName}:{record.lineno}')


def custom_exception_hook(notifier: Notifier):
    def handler(
            exception_type: type,
            exception_value: Exception,
            traceb,
            attributes: dict = None):
        except_file = traceback.TracebackException(exception_type,
                                                   exception_value,
                                                   traceb).stack[-1].filename
        notifier_params = {
            'subject': f'{os.path.basename(except_file)} Exception: {exception_type.__name__}: {exception_value}',
            'message': ''.join(traceback.format_tb(traceb)),
            'attributes': attributes
        }

        notifier.notify(**notifier_params)
        sys.__excepthook__(exception_type, exception_value, traceb)
    return handler



import os
import boto3

SNS_TOPIC_ARN = os.environ.get('SNS_TOPIC_ARN')


class Notifier():
    def __init__(self, client=None, sns_topic_arn: str = ""):
        assert SNS_TOPIC_ARN or sns_topic_arn, 'SNS_TOPIC_ARN ENV VAR MUST BE SET'
        self.client = client if client else boto3.Session().client('sns')
        self.sns_topic = sns_topic_arn if sns_topic_arn else SNS_TOPIC_ARN

    def notify(self, message: str, subject: str = "", attributes: dict = None):
        if not attributes:
            attributes = {}
        message_attributes = {
            key: {
                'DataType': 'String',
                'StringValue': value
            } for key, value in attributes.items()
        }
        self.client.publish(
            TopicArn=self.sns_topic,
            Message=self._validate_message(message),
            Subject=self._validate_subject(subject),
            MessageAttributes=message_attributes
        )

    @staticmethod
    def _validate_subject(subject: str) -> str:
        '''Validate a message subject by asserting it is of type 'str' and
        < 100 characters. if greater, truncates the message to 96 characters
        with a trailing elipsis to indicate the string was truncated
        '''
        assert isinstance(subject, str), f'Subject must be a string! Got: {type(subject)}'
        if len(subject) > 100:
            subject = subject[:96] + ' ...'
        return subject

    @staticmethod
    def _validate_message(message) -> str:
        '''Validate a message body by asserting it is of type 'str'
        and < 256kb. if greater, the message is split in half, and only the
        first 128kb-3 and last 128kb-3 are sent, seperated by "\n...\n" to
        indicate the message was truncated
        '''
        assert isinstance(message, str), f'Message body must be a string! Got: {type(message)}'
        if len(message.encode('utf-8')) > 256 * 1000:
            start = message.encode('utf-8')[:128 * 1000 - 3].decode('utf-8')
            end = message.encode('utf-8')[-128 * 1000 + 3:].decode('utf-8')
            message = start + '\n...\n' + end
        return message
